@@ -14,6 +14,8 @@
 #include "anim.h"
 #include "main.h"
 #include "tools.h"
+#include "events.h"
+#include "screens.h"
 
 
 /* values for global toon animation definition */
@@ -196,8 +198,8 @@ struct AnimClassGameMode
 /* forward declaration for internal use */
 static void HandleGlobalAnim(int, int);
 static void DoAnimationExt(void);
-static void ResetGlobalAnim_Clickable();
-static void ResetGlobalAnim_Clicked();
+static void ResetGlobalAnim_Clickable(void);
+static void ResetGlobalAnim_Clicked(void);
 
 static struct GlobalAnimControlInfo global_anim_ctrl[NUM_GAME_MODES];
 
@@ -206,6 +208,7 @@ static unsigned int anim_sync_frame = 0;
 static int game_mode_anim_classes[NUM_GAME_MODES];
 static int anim_class_game_modes[NUM_ANIM_CLASSES];
 
+static int anim_status_last_before_fading = GAME_MODE_DEFAULT;
 static int anim_status_last = GAME_MODE_DEFAULT;
 static int anim_classes_last = ANIM_CLASS_NONE;
 
@@ -322,7 +325,7 @@ static int compareGlobalAnimMainControlInfo(const void *obj1, const void *obj2)
   return compare_result;
 }
 
-static void InitToonControls()
+static void InitToonControls(void)
 {
   int mode_nr_toons = GAME_MODE_PSEUDO_TOONS;
   struct GlobalAnimControlInfo *ctrl = &global_anim_ctrl[mode_nr_toons];
@@ -404,7 +407,7 @@ static void InitToonControls()
   ctrl->num_anims++;
 }
 
-void InitGlobalAnimControls()
+static void InitGlobalAnimControls(void)
 {
   int i, m, a, p;
   int mode_nr, anim_nr, part_nr;
@@ -560,16 +563,17 @@ void InitGlobalAnimControls()
     anim_class_game_modes[anim_class_game_modes_list[i].class_bit] =
       anim_class_game_modes_list[i].game_mode;
 
+  anim_status_last_before_fading = GAME_MODE_LOADING;
   anim_status_last = GAME_MODE_LOADING;
   anim_classes_last = ANIM_CLASS_NONE;
 }
 
-void InitGlobalAnimations()
+void InitGlobalAnimations(void)
 {
   InitGlobalAnimControls();
 }
 
-void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
+static void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
 {
   Bitmap *fade_bitmap =
     (drawing_target == DRAW_TO_FADE_SOURCE ? gfx.fade_bitmap_source :
@@ -598,15 +602,23 @@ void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
     if (drawing_target == DRAW_TO_FADE_TARGET)
       after_fading = TRUE;
 
+    // special case: changing from/to this screen is done without fading
+    if (global.anim_status == GAME_MODE_PSEUDO_TYPENAME ||
+	anim_status_last   == GAME_MODE_PSEUDO_TYPENAME)
+      after_fading = TRUE;
+
     // ---------- part 1 ------------------------------------------------------
     // start or stop global animations by change of game mode
     // (special handling of animations for "current screen" and "all screens")
 
-    // stop animations for last screen
-    game_mode_anim_action[anim_status_last] = ANIM_STOP;
+    if (global.anim_status_next != anim_status_last_before_fading)
+    {
+      // stop animations for last screen before fading to new screen
+      game_mode_anim_action[anim_status_last] = ANIM_STOP;
 
-    // start animations for current screen
-    game_mode_anim_action[global.anim_status] = ANIM_START;
+      // start animations for current screen after fading to new screen
+      game_mode_anim_action[global.anim_status] = ANIM_START;
+    }
 
     // start animations for all screens after loading new artwork set
     if (anim_status_last == GAME_MODE_LOADING)
@@ -635,7 +647,10 @@ void DrawGlobalAnimationsExt(int drawing_target, int drawing_stage)
     if (drawing_target == DRAW_TO_SCREEN)
     {
       if (after_fading)
+      {
 	anim_classes_last = anim_classes_next;
+	anim_status_last_before_fading = global.anim_status;
+      }
 
       anim_status_last = global.anim_status;
 
@@ -791,7 +806,7 @@ void DrawGlobalAnimations(int drawing_target, int drawing_stage)
     ResetGlobalAnim_Clicked();
 }
 
-boolean SetGlobalAnimPart_Viewport(struct GlobalAnimPartControlInfo *part)
+static boolean SetGlobalAnimPart_Viewport(struct GlobalAnimPartControlInfo *part)
 {
   int viewport_x;
   int viewport_y;
@@ -902,7 +917,10 @@ static void PlayGlobalAnimMusic(struct GlobalAnimPartControlInfo *part)
   if (!setup.sound_music)
     return;
 
-  PlayMusic(music);
+  if (IS_LOOP_MUSIC(music))
+    PlayMusicLoop(music);
+  else
+    PlayMusic(music);
 
 #if 0
   printf("::: PLAY MUSIC %d.%d.%d: %d\n",
@@ -941,6 +959,28 @@ static void StopGlobalAnimSoundAndMusic(struct GlobalAnimPartControlInfo *part)
   StopGlobalAnimMusic(part);
 }
 
+static void PlayGlobalAnimSoundIfLoop(struct GlobalAnimPartControlInfo *part)
+{
+  // when drawing animations to fading buffer, do not play sounds
+  if (drawing_to_fading_buffer)
+    return;
+
+  // loop sounds only expire when playing
+  if (game_status != GAME_MODE_PLAYING)
+    return;
+
+  // check if any sound is defined for this animation part
+  if (part->sound == SND_UNDEFINED)
+    return;
+
+  // normal (non-loop) sounds do not expire when playing
+  if (!IS_LOOP_SOUND(part->sound))
+    return;
+
+  // prevent expiring loop sounds when playing
+  PlayGlobalAnimSound(part);
+}
+
 static boolean isClickablePart(struct GlobalAnimPartControlInfo *part, int mask)
 {
   struct GraphicInfo *c = &part->control_info;
@@ -990,7 +1030,13 @@ static boolean isClickedPart(struct GlobalAnimPartControlInfo *part,
   return TRUE;
 }
 
-int HandleGlobalAnim_Part(struct GlobalAnimPartControlInfo *part, int state)
+static boolean clickConsumed(struct GlobalAnimPartControlInfo *part)
+{
+  return (part->control_info.style & STYLE_PASSTHROUGH ? FALSE : TRUE);
+}
+
+static int HandleGlobalAnim_Part(struct GlobalAnimPartControlInfo *part,
+				 int state)
 {
   struct GlobalAnimControlInfo *ctrl = &global_anim_ctrl[part->mode_nr];
   struct GlobalAnimMainControlInfo *anim = &ctrl->anim[part->anim_nr];
@@ -1206,6 +1252,9 @@ int HandleGlobalAnim_Part(struct GlobalAnimPartControlInfo *part, int state)
     return ANIM_STATE_WAITING;
   }
 
+  // special case to prevent expiring loop sounds when playing
+  PlayGlobalAnimSoundIfLoop(part);
+
   if (!DelayReachedExt(&part->step_delay, part->step_delay_value,
 		       anim_sync_frame))
     return ANIM_STATE_RUNNING;
@@ -1231,11 +1280,14 @@ int HandleGlobalAnim_Part(struct GlobalAnimPartControlInfo *part, int state)
   return ANIM_STATE_RUNNING;
 }
 
-void HandleGlobalAnim_Main(struct GlobalAnimMainControlInfo *anim, int action)
+static void HandleGlobalAnim_Main(struct GlobalAnimMainControlInfo *anim,
+				  int action)
 {
   struct GlobalAnimPartControlInfo *part;
   struct GraphicInfo *c = &anim->control_info;
+  int num_parts = anim->num_parts + (anim->has_base ? 1 : 0);
   int state, active_part_nr;
+  int i;
 
 #if 0
   printf("::: HandleGlobalAnim_Main: %d, %d => %d\n",
@@ -1276,13 +1328,8 @@ void HandleGlobalAnim_Main(struct GlobalAnimMainControlInfo *anim, int action)
     case ANIM_STOP:
       anim->state = ANIM_STATE_INACTIVE;
 
-      {
-	int num_parts = anim->num_parts + (anim->has_base ? 1 : 0);
-	int i;
-
-	for (i = 0; i < num_parts; i++)
-	  StopGlobalAnimSoundAndMusic(&anim->part[i]);
-      }
+      for (i = 0; i < num_parts; i++)
+	StopGlobalAnimSoundAndMusic(&anim->part[i]);
 
       return;
 
@@ -1292,9 +1339,6 @@ void HandleGlobalAnim_Main(struct GlobalAnimMainControlInfo *anim, int action)
 
   if (c->anim_mode & ANIM_ALL || anim->num_parts == 0)
   {
-    int num_parts = anim->num_parts + (anim->has_base ? 1 : 0);
-    int i;
-
 #if 0
     printf("::: HandleGlobalAnim_Main: %d, %d => %d\n",
 	   anim->mode_nr, anim->nr, num_parts);
@@ -1346,6 +1390,11 @@ void HandleGlobalAnim_Main(struct GlobalAnimMainControlInfo *anim, int action)
 
   part = &anim->part[anim->active_part_nr];
 
+  // first set all animation parts to "inactive", ...
+  for (i = 0; i < num_parts; i++)
+    anim->part[i].state = ANIM_STATE_INACTIVE;
+
+  // ... then set current animation parts to "running"
   part->state = ANIM_STATE_RUNNING;
 
   anim->state = HandleGlobalAnim_Part(part, anim->state);
@@ -1384,7 +1433,7 @@ void HandleGlobalAnim_Main(struct GlobalAnimMainControlInfo *anim, int action)
   anim->last_active_part_nr = active_part_nr;
 }
 
-void HandleGlobalAnim_Mode(struct GlobalAnimControlInfo *ctrl, int action)
+static void HandleGlobalAnim_Mode(struct GlobalAnimControlInfo *ctrl, int action)
 {
   int i;
 
@@ -1406,7 +1455,7 @@ static void HandleGlobalAnim(int action, int game_mode)
   HandleGlobalAnim_Mode(&global_anim_ctrl[game_mode], action);
 }
 
-static void DoAnimationExt()
+static void DoAnimationExt(void)
 {
   int i;
 
@@ -1426,7 +1475,25 @@ static void DoAnimationExt()
 #endif
 }
 
-static void InitGlobalAnim_Clickable()
+static boolean DoGlobalAnim_EventAction(struct GlobalAnimPartControlInfo *part)
+{
+  int anim_event_action = part->control_info.anim_event_action;
+
+  if (anim_event_action == -1)
+    return FALSE;
+
+  boolean action_executed = (DoGadgetAction(anim_event_action) ||
+			     DoScreenAction(anim_event_action) ||
+			     DoKeysymAction(anim_event_action));
+
+  // check if further actions are allowed to be executed
+  if (part->control_info.style & STYLE_MULTIPLE_ACTIONS)
+    return FALSE;
+
+  return action_executed;
+}
+
+static void InitGlobalAnim_Clickable(void)
 {
   int mode_nr;
 
@@ -1452,20 +1519,25 @@ static void InitGlobalAnim_Clickable()
 
 static boolean InitGlobalAnim_Clicked(int mx, int my, boolean clicked)
 {
+  boolean anything_clicked = FALSE;
   boolean any_part_clicked = FALSE;
+  boolean any_event_action = FALSE;
   int mode_nr;
 
-  for (mode_nr = 0; mode_nr < NUM_GAME_MODES; mode_nr++)
+  // check game modes in reverse draw order (to stop when clicked)
+  for (mode_nr = NUM_GAME_MODES - 1; mode_nr >= 0; mode_nr--)
   {
     struct GlobalAnimControlInfo *ctrl = &global_anim_ctrl[mode_nr];
     int anim_nr;
 
-    for (anim_nr = 0; anim_nr < ctrl->num_anims; anim_nr++)
+    // check animations in reverse draw order (to stop when clicked)
+    for (anim_nr = ctrl->num_anims - 1; anim_nr >= 0; anim_nr--)
     {
       struct GlobalAnimMainControlInfo *anim = &ctrl->anim[anim_nr];
       int part_nr;
 
-      for (part_nr = 0; part_nr < anim->num_parts_all; part_nr++)
+      // check animation parts in reverse draw order (to stop when clicked)
+      for (part_nr = anim->num_parts_all - 1; part_nr >= 0; part_nr--)
       {
 	struct GlobalAnimPartControlInfo *part = &anim->part[part_nr];
 
@@ -1479,17 +1551,39 @@ static boolean InitGlobalAnim_Clicked(int mx, int my, boolean clicked)
 	if (!part->clickable)
 	  continue;
 
+	if (part->state != ANIM_STATE_RUNNING)
+	  continue;
+
+	// always handle "any" click events (clicking anywhere on screen) ...
 	if (isClickablePart(part, ANIM_EVENT_ANY))
-	  any_part_clicked = part->clicked = TRUE;
+	{
+	  part->clicked = TRUE;
+	  anything_clicked = clickConsumed(part);
+	}
+
+	// ... but only handle the first (topmost) clickable animation
+	if (any_part_clicked)
+	  continue;
 
 	if (isClickedPart(part, mx, my, clicked))
 	{
 #if 0
-	  printf("::: %d.%d CLICKED\n", anim_nr, part_nr);
+	  printf("::: %d.%d CLICKED [%d]\n", anim_nr, part_nr,
+		 part->control_info.anim_event_action);
 #endif
 
+	  // after executing event action, ignore any further actions
+	  if (!any_event_action && DoGlobalAnim_EventAction(part))
+	    any_event_action = TRUE;
+
+	  // determine if mouse clicks should be blocked from other animations
+	  any_part_clicked = clickConsumed(part);
+
 	  if (isClickablePart(part, ANIM_EVENT_SELF))
-	    any_part_clicked = part->clicked = TRUE;
+	  {
+	    part->clicked = TRUE;
+	    anything_clicked = clickConsumed(part);
+	  }
 
 	  // check if this click is defined to trigger other animations
 	  int gic_anim_nr = part->old_anim_nr + 1;	// X as in "anim_X"
@@ -1510,8 +1604,23 @@ static boolean InitGlobalAnim_Clicked(int mx, int my, boolean clicked)
 	    {
 	      struct GlobalAnimPartControlInfo *part2 = &anim2->part[part2_nr];
 
+	      if (part2->state != ANIM_STATE_RUNNING)
+		continue;
+
 	      if (isClickablePart(part2, mask))
-		any_part_clicked = part2->clicked = TRUE;
+	      {
+		part2->clicked = TRUE;
+		anything_clicked = clickConsumed(part);	// click was on "part"!
+
+#if 0
+		printf("::: %d.%d TRIGGER CLICKED [%d]\n", anim2_nr, part2_nr,
+		       part2->control_info.anim_event_action);
+#endif
+
+		// after executing event action, ignore any further actions
+		if (!any_event_action && DoGlobalAnim_EventAction(part2))
+		  any_event_action = TRUE;
+	      }
 
 #if 0
 	      struct GraphicInfo *c = &part2->control_info;
@@ -1532,15 +1641,15 @@ static boolean InitGlobalAnim_Clicked(int mx, int my, boolean clicked)
     }
   }
 
-  return any_part_clicked;
+  return (anything_clicked || any_event_action);
 }
 
-static void ResetGlobalAnim_Clickable()
+static void ResetGlobalAnim_Clickable(void)
 {
   InitGlobalAnim_Clickable();
 }
 
-static void ResetGlobalAnim_Clicked()
+static void ResetGlobalAnim_Clicked(void)
 {
   InitGlobalAnim_Clicked(-1, -1, FALSE);
 }
