@@ -15,33 +15,33 @@
 #include "conftime.h"
 
 
-char *getSourceDateString()
+char *getSourceDateString(void)
 {
   return SOURCE_DATE_STRING;
 }
 
-char *getProgramTitleString()
+char *getProgramTitleString(void)
 {
   return program.program_title;
 }
 
-char *getProgramRealVersionString()
+char *getProgramRealVersionString(void)
 {
   static char program_version_string[32];
 
   sprintf(program_version_string, "%d.%d.%d.%d%s",
-	  PROGRAM_VERSION_MAJOR, PROGRAM_VERSION_MINOR, PROGRAM_VERSION_PATCH,
-	  PROGRAM_VERSION_BUILD, PROGRAM_VERSION_EXTRA);
+	  PROGRAM_VERSION_SUPER, PROGRAM_VERSION_MAJOR, PROGRAM_VERSION_MINOR,
+	  PROGRAM_VERSION_PATCH, PROGRAM_VERSION_EXTRA);
 
   return program_version_string;
 }
 
-char *getProgramVersionString()
+char *getProgramVersionString(void)
 {
   return program.version_string;
 }
 
-char *getProgramInitString()
+char *getProgramInitString(void)
 {
   static char *program_init_string = NULL;
 
@@ -57,7 +57,43 @@ char *getProgramInitString()
   return program_init_string;
 }
 
-char *getWindowTitleString()
+char *getConfigProgramTitleString(void)
+{
+  TreeInfo *graphics_current =
+    getArtworkTreeInfoForUserLevelSet(ARTWORK_TYPE_GRAPHICS);
+
+  return (leveldir_current->program_title ?
+	  leveldir_current->program_title :
+	  graphics_current->program_title ?
+	  graphics_current->program_title :
+	  setup.internal.program_title);
+}
+
+char *getConfigProgramCopyrightString(void)
+{
+  TreeInfo *graphics_current =
+    getArtworkTreeInfoForUserLevelSet(ARTWORK_TYPE_GRAPHICS);
+
+  return (leveldir_current->program_copyright ?
+	  leveldir_current->program_copyright :
+	  graphics_current->program_copyright ?
+	  graphics_current->program_copyright :
+	  setup.internal.program_copyright);
+}
+
+char *getConfigProgramCompanyString(void)
+{
+  TreeInfo *graphics_current =
+    getArtworkTreeInfoForUserLevelSet(ARTWORK_TYPE_GRAPHICS);
+
+  return (leveldir_current->program_company ?
+	  leveldir_current->program_company :
+	  graphics_current->program_company ?
+	  graphics_current->program_company :
+	  setup.internal.program_company);
+}
+
+char *getWindowTitleString(void)
 {
   static char *window_title_string = NULL;
 
