@@ -164,7 +164,7 @@ boolean NextValidEvent(Event *event)
   return FALSE;
 }
 
-void HandleEvents()
+static void HandleEvents(void)
 {
   Event event;
   unsigned int event_frame_delay = 0;
@@ -283,7 +283,7 @@ void HandleOtherEvents(Event *event)
   }
 }
 
-void HandleMouseCursor()
+static void HandleMouseCursor(void)
 {
   if (game_status == GAME_MODE_TITLE)
   {
@@ -347,7 +347,26 @@ void EventLoop(void)
   }
 }
 
-void ClearEventQueue()
+void ClearAutoRepeatKeyEvents(void)
+{
+#if defined(TARGET_SDL2)
+  while (PendingEvent())
+  {
+    Event next_event;
+
+    PeekEvent(&next_event);
+
+    /* if event is repeated key press event, remove it from event queue */
+    if (next_event.type == EVENT_KEYPRESS &&
+	next_event.key.repeat)
+      WaitEvent(&next_event);
+    else
+      break;
+  }
+#endif
+}
+
+void ClearEventQueue(void)
 {
   Event event;
 
@@ -377,14 +396,14 @@ void ClearEventQueue()
   }
 }
 
-void ClearPlayerMouseAction()
+static void ClearPlayerMouseAction(void)
 {
   local_player->mouse_action.lx = 0;
   local_player->mouse_action.ly = 0;
   local_player->mouse_action.button = 0;
 }
 
-void ClearPlayerAction()
+void ClearPlayerAction(void)
 {
   int i;
 
@@ -397,7 +416,7 @@ void ClearPlayerAction()
   ClearPlayerMouseAction();
 }
 
-void SetPlayerMouseAction(int mx, int my, int button)
+static void SetPlayerMouseAction(int mx, int my, int button)
 {
   int lx = getLevelFromScreenX(mx);
   int ly = getLevelFromScreenY(my);
@@ -425,7 +444,7 @@ void SetPlayerMouseAction(int mx, int my, int button)
   SetTileCursorXY(lx, ly);
 }
 
-void SleepWhileUnmapped()
+void SleepWhileUnmapped(void)
 {
   boolean window_unmapped = TRUE;
 
@@ -636,10 +655,30 @@ void HandleWindowEvent(WindowEvent *event)
       if (new_display_width  != video.display_width ||
 	  new_display_height != video.display_height)
       {
+	int nr = GRID_ACTIVE_NR();	// previous screen orientation
+
 	video.display_width  = new_display_width;
 	video.display_height = new_display_height;
 
 	SDLSetScreenProperties();
+
+	// check if screen orientation has changed (should always be true here)
+	if (nr != GRID_ACTIVE_NR())
+	{
+	  int x, y;
+
+	  if (game_status == GAME_MODE_SETUP)
+	    RedrawSetupScreenAfterScreenRotation(nr);
+
+	  nr = GRID_ACTIVE_NR();
+
+	  overlay.grid_xsize = setup.touch.grid_xsize[nr];
+	  overlay.grid_ysize = setup.touch.grid_ysize[nr];
+
+	  for (x = 0; x < MAX_GRID_XSIZE; x++)
+	    for (y = 0; y < MAX_GRID_YSIZE; y++)
+	      overlay.grid_button[x][y] = setup.touch.grid_button[nr][x][y];
+	}
       }
     }
 #endif
@@ -656,8 +695,21 @@ static struct
   Key key;
 } touch_info[NUM_TOUCH_FINGERS];
 
-void HandleFingerEvent_VirtualButtons(FingerEvent *event)
+static void HandleFingerEvent_VirtualButtons(FingerEvent *event)
 {
+#if 1
+  int x = event->x * overlay.grid_xsize;
+  int y = event->y * overlay.grid_ysize;
+  int grid_button = overlay.grid_button[x][y];
+  int grid_button_action = GET_ACTION_FROM_GRID_BUTTON(grid_button);
+  Key key = (grid_button == CHAR_GRID_BUTTON_LEFT  ? setup.input[0].key.left :
+	     grid_button == CHAR_GRID_BUTTON_RIGHT ? setup.input[0].key.right :
+	     grid_button == CHAR_GRID_BUTTON_UP    ? setup.input[0].key.up :
+	     grid_button == CHAR_GRID_BUTTON_DOWN  ? setup.input[0].key.down :
+	     grid_button == CHAR_GRID_BUTTON_SNAP  ? setup.input[0].key.snap :
+	     grid_button == CHAR_GRID_BUTTON_DROP  ? setup.input[0].key.drop :
+	     KSYM_UNDEFINED);
+#else
   float ypos = 1.0 - 1.0 / 3.0 * video.display_width / video.display_height;
   float event_x = (event->x);
   float event_y = (event->y - ypos) / (1 - ypos);
@@ -680,6 +732,7 @@ void HandleFingerEvent_VirtualButtons(FingerEvent *event)
 	     event_y > 2.0 / 3.0 && event_y < 1 ?
 	     setup.input[0].key.down :
 	     KSYM_UNDEFINED);
+#endif
   int key_status = (event->type == EVENT_FINGERRELEASE ? KEY_RELEASED :
 		    KEY_PRESSED);
   char *key_status_name = (key_status == KEY_RELEASED ? "KEY_RELEASED" :
@@ -692,6 +745,11 @@ void HandleFingerEvent_VirtualButtons(FingerEvent *event)
   Error(ERR_DEBUG, "::: key '%s' was '%s' [fingerId: %lld]",
 	getKeyNameFromKey(key), key_status_name, event->fingerId);
 
+  if (key_status == KEY_PRESSED)
+    overlay.grid_button_action |= grid_button_action;
+  else
+    overlay.grid_button_action &= ~grid_button_action;
+
   // check if we already know this touch event's finger id
   for (i = 0; i < NUM_TOUCH_FINGERS; i++)
   {
@@ -797,7 +855,7 @@ void HandleFingerEvent_VirtualButtons(FingerEvent *event)
   }
 }
 
-void HandleFingerEvent_WipeGestures(FingerEvent *event)
+static void HandleFingerEvent_WipeGestures(FingerEvent *event)
 {
   static Key motion_key_x = KSYM_UNDEFINED;
   static Key motion_key_y = KSYM_UNDEFINED;
@@ -1331,6 +1389,8 @@ static void HandleButtonOrFinger(int mx, int my, int button)
       HandleButtonOrFinger_WipeGestures_MM(mx, my, button);
     else if (strEqual(setup.touch.control_type, TOUCH_CONTROL_FOLLOW_FINGER))
       HandleButtonOrFinger_FollowFinger_MM(mx, my, button);
+    else if (strEqual(setup.touch.control_type, TOUCH_CONTROL_VIRTUAL_BUTTONS))
+      SetPlayerMouseAction(mx, my, button);	/* special case */
   }
   else
   {
@@ -1341,7 +1401,7 @@ static void HandleButtonOrFinger(int mx, int my, int button)
 
 #if defined(TARGET_SDL2)
 
-static boolean checkTextInputKeyModState()
+static boolean checkTextInputKeyModState(void)
 {
   // when playing, only handle raw key events and ignore text input
   if (game_status == GAME_MODE_PLAYING)
@@ -1515,18 +1575,18 @@ void HandleButton(int mx, int my, int button, int button_nr)
      strEqual(setup.touch.control_type, TOUCH_CONTROL_FOLLOW_FINGER));
 #endif
 
-  if (handle_gadgets && HandleGadgets(mx, my, button))
-  {
-    /* do not handle this button event anymore */
-    mx = my = -32;	/* force mouse event to be outside screen tiles */
-  }
-
   if (HandleGlobalAnimClicks(mx, my, button))
   {
     /* do not handle this button event anymore */
     return;		/* force mouse event not to be handled at all */
   }
 
+  if (handle_gadgets && HandleGadgets(mx, my, button))
+  {
+    /* do not handle this button event anymore */
+    mx = my = -32;	/* force mouse event to be outside screen tiles */
+  }
+
   if (button_hold && game_status == GAME_MODE_PLAYING && tape.pausing)
     return;
 
@@ -1636,6 +1696,11 @@ static void HandleKeysSpecial(Key key)
     {
       InsertSolutionTape();
     }
+    else if (is_string_suffix(cheat_input, ":play-solution-tape") ||
+	     is_string_suffix(cheat_input, ":pst"))
+    {
+      PlaySolutionTape();
+    }
     else if (is_string_suffix(cheat_input, ":reload-graphics") ||
 	     is_string_suffix(cheat_input, ":rg"))
     {
@@ -1716,6 +1781,15 @@ static void HandleKeysSpecial(Key key)
       DumpBrush_Small();
     }
   }
+
+  /* special key shortcuts for all game modes */
+  if (is_string_suffix(cheat_input, ":dump-event-actions") ||
+      is_string_suffix(cheat_input, ":dea") ||
+      is_string_suffix(cheat_input, ":DEA"))
+  {
+    DumpGadgetIdentifiers();
+    DumpScreenIdentifiers();
+  }
 }
 
 void HandleKeysDebug(Key key)
@@ -1978,7 +2052,7 @@ void HandleKey(Key key, int key_status)
   if (game_status == GAME_MODE_MAIN &&
       (key == setup.shortcut.toggle_pause || key == KSYM_space))
   {
-    StartGameActions(options.network, setup.autorecord, level.random_seed);
+    StartGameActions(network.enabled, setup.autorecord, level.random_seed);
 
     return;
   }
@@ -2041,6 +2115,10 @@ void HandleKey(Key key, int key_status)
     case GAME_MODE_SETUP:
     case GAME_MODE_INFO:
     case GAME_MODE_SCORES:
+
+      if (anyTextGadgetActiveOrJustFinished && key != KSYM_Escape)
+	break;
+
       switch (key)
       {
 	case KSYM_space:
@@ -2143,7 +2221,7 @@ void HandleKey(Key key, int key_status)
   HandleKeysDebug(key);
 }
 
-void HandleNoEvent()
+void HandleNoEvent(void)
 {
   HandleMouseCursor();
 
@@ -2155,7 +2233,7 @@ void HandleNoEvent()
   }
 }
 
-void HandleEventActions()
+void HandleEventActions(void)
 {
   // if (button_status && game_status != GAME_MODE_PLAYING)
   if (button_status && (game_status != GAME_MODE_PLAYING ||
@@ -2169,10 +2247,8 @@ void HandleEventActions()
     HandleJoystick();
   }
 
-#if defined(NETWORK_AVALIABLE)
-  if (options.network)
+  if (network.enabled)
     HandleNetworking();
-#endif
 
   switch (game_status)
   {
@@ -2220,7 +2296,7 @@ static void HandleTileCursor(int dx, int dy, int button)
   }
 }
 
-static int HandleJoystickForAllPlayers()
+static int HandleJoystickForAllPlayers(void)
 {
   int i;
   int result = 0;
@@ -2253,7 +2329,7 @@ static int HandleJoystickForAllPlayers()
   return result;
 }
 
-void HandleJoystick()
+void HandleJoystick(void)
 {
   static unsigned int joytest_delay = 0;
   static unsigned int joytest_delay_value = GADGET_FRAME_DELAY;
@@ -2323,6 +2399,9 @@ void HandleJoystick()
     case GAME_MODE_INFO:
     case GAME_MODE_SCORES:
     {
+      if (anyTextGadgetActive())
+	break;
+
       if (game_status == GAME_MODE_TITLE)
 	HandleTitleScreen(0,0,dx,dy, newbutton ? MB_MENU_CHOICE : MB_MENU_MARK);
       else if (game_status == GAME_MODE_MAIN)
@@ -2379,24 +2458,38 @@ void HandleJoystick()
 void HandleSpecialGameControllerButtons(Event *event)
 {
 #if defined(TARGET_SDL2)
+  int key_status;
+  Key key;
+
   switch (event->type)
   {
     case SDL_CONTROLLERBUTTONDOWN:
-      if (event->cbutton.button == SDL_CONTROLLER_BUTTON_START)
-	HandleKey(KSYM_space, KEY_PRESSED);
-      else if (event->cbutton.button == SDL_CONTROLLER_BUTTON_BACK)
-	HandleKey(KSYM_Escape, KEY_PRESSED);
-
+      key_status = KEY_PRESSED;
       break;
 
     case SDL_CONTROLLERBUTTONUP:
-      if (event->cbutton.button == SDL_CONTROLLER_BUTTON_START)
-	HandleKey(KSYM_space, KEY_RELEASED);
-      else if (event->cbutton.button == SDL_CONTROLLER_BUTTON_BACK)
-	HandleKey(KSYM_Escape, KEY_RELEASED);
-
+      key_status = KEY_RELEASED;
       break;
+
+    default:
+      return;
   }
+
+  switch (event->cbutton.button)
+  {
+    case SDL_CONTROLLER_BUTTON_START:
+      key = KSYM_space;
+      break;
+
+    case SDL_CONTROLLER_BUTTON_BACK:
+      key = KSYM_Escape;
+      break;
+
+    default:
+      return;
+  }
+
+  HandleKey(key, key_status);
 #endif
 }
 
@@ -2429,3 +2522,18 @@ void HandleSpecialGameControllerKeys(Key key, int key_status)
 #endif
 #endif
 }
+
+boolean DoKeysymAction(int keysym)
+{
+  if (keysym < 0)
+  {
+    Key key = (Key)(-keysym);
+
+    HandleKey(key, KEY_PRESSED);
+    HandleKey(key, KEY_RELEASED);
+
+    return TRUE;
+  }
+
+  return FALSE;
+}
