@@ -665,6 +665,11 @@ struct GlobalInfo_EM
 struct GameInfo_EM
 {
   boolean any_player_moving;
+  boolean any_player_snapping;
+
+  boolean use_single_button;
+  boolean use_snap_key_bug;
+
   int last_moving_player;
   int last_player_direction[MAX_PLAYERS];
 };
@@ -677,6 +682,9 @@ struct LevelInfo_EM
 
   struct LEVEL *lev;
   struct PLAYER *ply[MAX_PLAYERS];
+
+  /* used for runtime values */
+  struct GameInfo_EM *game_em;
 };
 
 struct GraphicInfo_EM
@@ -690,6 +698,7 @@ struct GraphicInfo_EM
   Bitmap *crumbled_bitmap;
   int crumbled_src_x, crumbled_src_y;
   int crumbled_border_size;
+  int crumbled_tile_size;
 
   boolean has_crumbled_graphics;
   boolean preserve_background;
@@ -719,30 +728,38 @@ struct EngineSnapshotInfo_EM
 /* ------------------------------------------------------------------------- */
 
 extern struct GlobalInfo_EM global_em_info;
+extern struct GameInfo_EM game_em;
 extern struct LevelInfo_EM native_em_level;
 extern struct GraphicInfo_EM graphic_info_em_object[TILE_MAX][8];
 extern struct GraphicInfo_EM graphic_info_em_player[MAX_PLAYERS][SPR_MAX][8];
 extern struct EngineSnapshotInfo_EM engine_snapshot_em;
 
-extern void em_open_all();
-extern void em_close_all();
+extern void em_open_all(void);
+extern void em_close_all(void);
 
-extern void InitGfxBuffers_EM();
+extern void InitGfxBuffers_EM(void);
 
-extern void InitGameEngine_EM();
+extern void InitGameEngine_EM(void);
 extern void GameActions_EM(byte *, boolean);
 
 extern unsigned int InitEngineRandom_EM(int);
 
-extern void setLevelInfoToDefaults_EM();
+extern void setLevelInfoToDefaults_EM(void);
 extern boolean LoadNativeLevel_EM(char *, boolean);
 
+extern int getFieldbufferOffsetX_EM(void);
+extern int getFieldbufferOffsetY_EM(void);
+
 extern void BackToFront_EM(void);
 extern void BlitScreenToBitmap_EM(Bitmap *);
 extern void RedrawPlayfield_EM(boolean);
-extern void DrawGameDoorValues_EM();
 
-extern void LoadEngineSnapshotValues_EM();
-extern void SaveEngineSnapshotValues_EM();
+extern void LoadEngineSnapshotValues_EM(void);
+extern void SaveEngineSnapshotValues_EM(void);
+
+extern boolean checkIfAllPlayersFitToScreen(void);
+
+extern void tab_generate(void);
+extern void tab_generate_graphics_info_em(void);
 
 #endif	/* EXPORT_H */
