@@ -7,19 +7,43 @@
 #include <math.h>
 
 
-long mScrollX, mScrollY;
-long mScrollX_last, mScrollY_last;
+int mScrollX, mScrollY;
+int mScrollX_last, mScrollY_last;
 
-#if 1
-long ScreenBuffer[2 + MAX_PLAYFIELD_WIDTH + 2][2 + MAX_PLAYFIELD_HEIGHT + 2];
-boolean redraw[2 + MAX_PLAYFIELD_WIDTH + 2][2 + MAX_PLAYFIELD_HEIGHT + 2];
-#else
-long ScreenBuffer[MAX_BUF_XSIZE][MAX_BUF_YSIZE];
-boolean redraw[MAX_BUF_XSIZE][MAX_BUF_YSIZE];
-#endif
+int ScreenBuffer[2 + MAX_PLAYFIELD_WIDTH + 2][2 + MAX_PLAYFIELD_HEIGHT + 2];
 
 
-void RestorePlayfield()
+int getFieldbufferOffsetX_SP(void)
+{
+  int px = 2 * TILEX + (mScrollX - mScrollX_last) % TILEX;
+
+  /* scroll correction for even number of visible tiles (half tile shifted) */
+  px += game_sp.scroll_xoffset;
+
+  if (ExplosionShakeMurphy != 0)
+    px += TILEX / 2 - GetSimpleRandom(TILEX + 1);
+
+  px = px * TILESIZE_VAR / TILESIZE;
+
+  return px;
+}
+
+int getFieldbufferOffsetY_SP(void)
+{
+  int py = 2 * TILEY + (mScrollY - mScrollY_last) % TILEY;
+
+  /* scroll correction for even number of visible tiles (half tile shifted) */
+  py += game_sp.scroll_yoffset;
+
+  if (ExplosionShakeMurphy != 0)
+    py += TILEY / 2 - GetSimpleRandom(TILEX + 1);
+
+  py = py * TILESIZE_VAR / TILESIZE;
+
+  return py;
+}
+
+void RestorePlayfield(void)
 {
   int x1 = mScrollX / TILEX - 2;
   int y1 = mScrollY / TILEY - 2;
@@ -51,18 +75,12 @@ static void ScrollPlayfield(int dx, int dy)
   int x, y;
 
   BlitBitmap(bitmap_db_field_sp, bitmap_db_field_sp,
-             TILEX * (dx == -1),
-             TILEY * (dy == -1),
-             (MAX_BUF_XSIZE * TILEX) - TILEX * (dx != 0),
-             (MAX_BUF_YSIZE * TILEY) - TILEY * (dy != 0),
-             TILEX * (dx == 1),
-             TILEY * (dy == 1));
-
-  /* when scrolling the whole playfield, do not redraw single tiles */
-  for (x = 0; x < MAX_BUF_XSIZE; x++)
-    for (y = 0; y < MAX_BUF_YSIZE; y++)
-      redraw[x][y] = FALSE;
-  redraw_tiles = 0;
+             TILEX_VAR * (dx == -1),
+             TILEY_VAR * (dy == -1),
+             (MAX_BUF_XSIZE * TILEX_VAR) - TILEX_VAR * (dx != 0),
+             (MAX_BUF_YSIZE * TILEY_VAR) - TILEY_VAR * (dy != 0),
+             TILEX_VAR * (dx == 1),
+             TILEY_VAR * (dy == 1));
 
   DrawFrameIfNeeded();
 
@@ -75,9 +93,9 @@ static void ScrollPlayfield(int dx, int dy)
 	int sx = x - x1;
 	int sy = y - y1;
 	int tsi = GetSI(x, y);
-	long id = ((PlayField16[tsi]) |
-		   (PlayField8[tsi] << 16) |
-		   (DisPlayField[tsi] << 24));
+	int id = ((PlayField16[tsi]) |
+		  (PlayField8[tsi] << 16) |
+		  (DisPlayField[tsi] << 24));
 
 	if ((dx == -1 && x == x2) ||
 	    (dx == +1 && x == x1) ||
@@ -106,13 +124,8 @@ static void ScrollPlayfieldIfNeededExt(boolean reset)
 
   if (mScrollX_last == -1 || mScrollY_last == -1)
   {
-#if 1
     mScrollX_last = (mScrollX / TILESIZE) * TILESIZE;
     mScrollY_last = (mScrollY / TILESIZE) * TILESIZE;
-#else
-    mScrollX_last = mScrollX;
-    mScrollY_last = mScrollY;
-#endif
 
     return;
   }
@@ -135,23 +148,37 @@ static void ScrollPlayfieldIfNeededExt(boolean reset)
   }
 }
 
-static void ScrollPlayfieldIfNeeded()
+static void ScrollPlayfieldIfNeeded(void)
 {
   ScrollPlayfieldIfNeededExt(FALSE);
 }
 
-void InitScrollPlayfield()
+void InitScrollPlayfield(void)
 {
   ScrollPlayfieldIfNeededExt(TRUE);
 }
 
+#define DEBUG_REDRAW	0
+
 void UpdatePlayfield(boolean force_redraw)
 {
   int x, y;
-#if 1
+
+#if DEBUG_REDRAW
   int num_redrawn = 0;
 #endif
 
+  if (force_redraw)
+  {
+    // force re-initialization of graphics status variables
+    for (y = DisplayMinY; y <= DisplayMaxY; y++)
+      for (x = DisplayMinX; x <= DisplayMaxX; x++)
+	GfxGraphic[x][y] = -1;
+
+    // force complete playfield redraw
+    DisplayLevel();
+  }
+
   for (y = DisplayMinY; y <= DisplayMaxY; y++)
   {
     for (x = DisplayMinX; x <= DisplayMaxX; x++)
@@ -161,10 +188,6 @@ void UpdatePlayfield(boolean force_redraw)
       int sync_frame = GfxFrame[x][y];
       boolean redraw = force_redraw;
 
-#if 0
-      redraw = TRUE;	// !!! TEST ONLY -- ALWAYS REDRAW !!!
-#endif
-
       if (graphic < 0)
       {
 	GfxGraphicLast[x][y] = GfxGraphic[x][y];
@@ -201,108 +224,38 @@ void UpdatePlayfield(boolean force_redraw)
 	int sx = x * StretchWidth;
 	int sy = y * StretchWidth;
 
-#if 0
-	printf("::: REDRAW (%d, %d): %d, %d\n", x, y, graphic, sync_frame);
-#endif
-
 	DDSpriteBuffer_BltImg(sx, sy, graphic, sync_frame);
 
-#if 1
+#if DEBUG_REDRAW
 	num_redrawn++;
 #endif
       }
     }
   }
 
-#if 0
+#if DEBUG_REDRAW
   printf("::: FRAME %d: %d redrawn\n", FrameCounter, num_redrawn);
 #endif
 }
 
-/* copy the entire screen to the window at the scroll position */
-
 void BlitScreenToBitmap_SP(Bitmap *target_bitmap)
 {
-  int px = 2 * TILEX + (mScrollX - mScrollX_last) % TILEX;
-  int py = 2 * TILEY + (mScrollY - mScrollY_last) % TILEY;
-  int sx, sy, sxsize, sysize;
-
-#if 0
-  printf("::: %d, %d / %d, %d / %ld, %ld (%ld, %ld) / %d, %d\n",
-	 MurphyScreenXPos, MurphyScreenYPos,
-	 ScreenScrollXPos, ScreenScrollYPos,
-	 mScrollX, mScrollY,
-	 mScrollX_last, mScrollY_last,
-	 px, py);
-#endif
+  /* copy playfield buffer to target bitmap at scroll position */
 
+  int px = getFieldbufferOffsetX_SP();
+  int py = getFieldbufferOffsetY_SP();
   int xsize = SXSIZE;
   int ysize = SYSIZE;
-  int full_xsize = (FieldWidth  - (menBorder ? 0 : 1)) * TILEX;
-  int full_ysize = (FieldHeight - (menBorder ? 0 : 1)) * TILEY;
-
-  sxsize = (full_xsize < xsize ? full_xsize : xsize);
-  sysize = (full_ysize < ysize ? full_ysize : ysize);
-  sx = SX + (full_xsize < xsize ? (xsize - full_xsize) / 2 : 0);
-  sy = SY + (full_ysize < ysize ? (ysize - full_ysize) / 2 : 0);
-
-  /* scroll correction for even number of visible tiles (half tile shifted) */
-  px += game_sp.scroll_xoffset;
-  py += game_sp.scroll_yoffset;
-
-#if 1
-  if (ExplosionShakeMurphy != 0)
-  {
-    px += TILEX / 2 - GetSimpleRandom(TILEX + 1);
-    py += TILEY / 2 - GetSimpleRandom(TILEX + 1);
-  }
-#endif
+  int full_xsize = (FieldWidth  - (menBorder ? 0 : 1)) * TILEX_VAR;
+  int full_ysize = (FieldHeight - (menBorder ? 0 : 1)) * TILEY_VAR;
+  int sx = SX + (full_xsize < xsize ? (xsize - full_xsize) / 2 : 0);
+  int sy = SY + (full_ysize < ysize ? (ysize - full_ysize) / 2 : 0);
+  int sxsize = (full_xsize < xsize ? full_xsize : xsize);
+  int sysize = (full_ysize < ysize ? full_ysize : ysize);
 
   BlitBitmap(bitmap_db_field_sp, target_bitmap, px, py, sxsize, sysize, sx, sy);
 }
 
-void BackToFront_SP(void)
-{
-  static boolean scrolling_last = FALSE;
-  int left = mScrollX / TILEX;
-  int top  = mScrollY / TILEY;
-  boolean scrolling = (mScrollX % TILEX != 0 || mScrollY % TILEY != 0);
-  int x, y;
-
-  SyncDisplay();
-
-  if (1 ||
-      redraw_tiles > REDRAWTILES_THRESHOLD || scrolling || scrolling_last)
-  {
-    BlitScreenToBitmap_SP(window);
-  }
-  else
-  {
-    for (x = 0; x < SCR_FIELDX; x++)
-    {
-      for (y = 0; y < SCR_FIELDY; y++)
-      {
-	int xx = (left + x) % MAX_BUF_XSIZE;
-	int yy = (top  + y) % MAX_BUF_YSIZE;
-
-	if (redraw[xx][yy])
-	  BlitBitmap(bitmap_db_field_sp, window,
-		     xx * TILEX, yy * TILEY, TILEX, TILEY,
-		     SX + x * TILEX, SY + y * TILEY);
-      }
-    }
-  }
-
-  FlushDisplay();
-
-  for (x = 0; x < MAX_BUF_XSIZE; x++)
-    for (y = 0; y < MAX_BUF_YSIZE; y++)
-      redraw[x][y] = FALSE;
-  redraw_tiles = 0;
-
-  scrolling_last = scrolling;
-}
-
 void DDScrollBuffer_ScrollTo(int X, int Y)
 {
   if (NoDisplayFlag)
@@ -339,12 +292,12 @@ void DDScrollBuffer_ScrollTowards(int X, int Y, double Step)
   ScrollPlayfieldIfNeeded();
 }
 
-void DDScrollBuffer_SoftScrollTo(int X, int Y, long TimeMS, int FPS)
+void DDScrollBuffer_SoftScrollTo(int X, int Y, int TimeMS, int FPS)
 {
   double dx, dY;
-  long dT, StepCount;
+  int StepCount;
   double T, tStep;
-  long oldX, oldY, maxD;
+  int oldX, oldY, maxD;
   static boolean AlreadyRunning = False;
 
   if (NoDisplayFlag)
@@ -366,7 +319,6 @@ void DDScrollBuffer_SoftScrollTo(int X, int Y, long TimeMS, int FPS)
   if (StepCount == 0)
     StepCount = 1;
 
-  dT = 1000 / FPS;
   tStep = (double)1 / StepCount;
   oldX = mScrollX;
   oldY = mScrollY;
