@@ -12,9 +12,9 @@ extern int mScrollX, mScrollY;
 extern int mScrollX_last, mScrollY_last;
 
 
-extern void InitScrollPlayfield();
+extern void InitScrollPlayfield(void);
 extern void UpdatePlayfield(boolean);
-extern void RestorePlayfield();
+extern void RestorePlayfield(void);
 
 extern void DDScrollBuffer_ScrollTo(int X, int Y);
 extern void DDScrollBuffer_ScrollTowards(int X, int Y, double Step);
