@@ -10,6 +10,6 @@
 extern int AnimationPosTable[SP_MAX_PLAYFIELD_SIZE];
 extern byte AnimationSubTable[SP_MAX_PLAYFIELD_SIZE];
 
-extern void subDoGameStuff();
+extern void subDoGameStuff(void);
 
 #endif /* DOGAMESTUFF_H */
