@@ -11,10 +11,6 @@
 
 /* helper functions for constructs not supported by C */
 
-#if 0
-extern void *REDIM_1D(int, int, int);
-#endif
-
-extern int MyGetTickCount();
+extern int MyGetTickCount(void);
 
 #endif /* VB_LIB_H */
