@@ -430,6 +430,12 @@ struct MouseCursorInfo
 #define KMOD_Meta		(KMOD_Meta_L    | KMOD_Meta_R)
 #define KMOD_Alt		(KMOD_Alt_L     | KMOD_Alt_R)
 
+/* this only contains "valid" key modifiers (and ignores keys like "NumLock") */
+#define KMOD_Valid		(KMOD_Shift   |	\
+				 KMOD_Control |	\
+				 KMOD_Meta    |	\
+				 KMOD_Alt)
+
 #if defined(TARGET_SDL2)
 #define KMOD_TextInput		(KMOD_Shift | KMOD_Alt_R)
 #endif
@@ -453,7 +459,8 @@ void SDLSetScreenProperties(void);
 #endif
 
 void SDLSetScreenRenderingMode(char *);
-void SDLRedrawWindow();
+void SDLSetScreenVsyncMode(char *);
+void SDLRedrawWindow(void);
 void SDLSetWindowTitle(void);
 
 void SDLLimitScreenUpdates(boolean);
@@ -484,17 +491,19 @@ void SDLSetMouseCursor(struct MouseCursorInfo *);
 void SDLOpenAudio(void);
 void SDLCloseAudio(void);
 
-void SDLNextEvent(Event *);
+void SDLWaitEvent(Event *);
 void SDLHandleWindowManagerEvent(Event *);
 
 void HandleJoystickEvent(Event *);
 void SDLInitJoysticks(void);
 boolean SDLReadJoystick(int, int *, int *, boolean *, boolean *);
 boolean SDLCheckJoystickOpened(int);
-void SDLClearJoystickState();
+void SDLClearJoystickState(void);
 boolean SDLOpenJoystick(int);
 void SDLCloseJoystick(int);
 
 void PrepareFadeBitmap(int);
 
+void Delay_WithScreenUpdates(unsigned int);
+
 #endif /* SDL_H */
