@@ -60,7 +60,8 @@
 
 #define SND_CTRL_PLAY_SOUND		(SND_CTRL_NONE)
 #define SND_CTRL_PLAY_LOOP		(SND_CTRL_LOOP)
-#define SND_CTRL_PLAY_MUSIC		(SND_CTRL_LOOP | SND_CTRL_MUSIC)
+#define SND_CTRL_PLAY_MUSIC		(SND_CTRL_MUSIC)
+#define SND_CTRL_PLAY_MUSIC_LOOP	(SND_CTRL_MUSIC | SND_CTRL_LOOP)
 
 #define SND_CTRL_FADE_SOUND		(SND_CTRL_FADE)
 #define SND_CTRL_FADE_MUSIC		(SND_CTRL_FADE | SND_CTRL_MUSIC)
@@ -102,10 +103,12 @@ void StartMixer(void);
 
 /* sound client functions */
 void PlayMusic(int);
+void PlayMusicLoop(int);
 void PlaySound(int);
 void PlaySoundStereo(int, int);
 void PlaySoundLoop(int);
 void PlaySoundMusic(int);
+void PlaySoundMusicLoop(int);
 void PlaySoundExt(int, int, int, int);
 void FadeMusic(void);
 void FadeSound(int);
@@ -117,15 +120,16 @@ void StopSounds(void);
 void StopSoundExt(int, int);
 void ExpireSoundLoops(boolean);
 
-int getSoundListSize();
-int getMusicListSize();
+int getSoundListSize(void);
+int getMusicListSize(void);
 struct FileInfo *getSoundListEntry(int);
 struct FileInfo *getMusicListEntry(int);
-char *getCurrentlyPlayingMusicFilename();
-int getSoundListPropertyMappingSize();
-int getMusicListPropertyMappingSize();
-struct PropertyMapping *getSoundListPropertyMapping();
-struct PropertyMapping *getMusicListPropertyMapping();
+char *getMusicInfoEntryFilename(int);
+char *getCurrentlyPlayingMusicFilename(void);
+int getSoundListPropertyMappingSize(void);
+int getMusicListPropertyMappingSize(void);
+struct PropertyMapping *getSoundListPropertyMapping(void);
+struct PropertyMapping *getMusicListPropertyMapping(void);
 void InitSoundList(struct ConfigInfo *, int, struct ConfigTypeInfo *,
 		   char **, char **, char **, char **, char **);
 void InitMusicList(struct ConfigInfo *, int, struct ConfigTypeInfo *,
