@@ -18,9 +18,9 @@
 #include <string.h>
 #include <sys/types.h>
 
-typedef int boolean;
 
 #if !defined(PLATFORM_WIN32)
+typedef int boolean;
 typedef unsigned char byte;
 #endif
 
@@ -64,8 +64,7 @@ typedef unsigned char byte;
 #define EVEN(a)		(((a) & 1) == 0)
 #endif
 
-#define SIZEOF_ARRAY(array, type)	(sizeof(array) / sizeof(type))
-#define SIZEOF_ARRAY_INT(array)		SIZEOF_ARRAY(array, int)
+#define ARRAY_SIZE(array)		(sizeof(array) / sizeof(array[0]))
 
 
 struct ListNode
