@@ -12,10 +12,16 @@
 #ifndef WINDOWS_H
 #define WINDOWS_H
 
+/* prevent including lots of conflicting macros */
+#define WIN32_LEAN_AND_MEAN
+
 #include <shlobj.h>
 
 
 /* some symbols are already defined on Windows */
+typedef int boolean_internal;
+#define boolean boolean_internal
+
 #define CreateBitmap CreateBitmap_internal
 #define GetPixel GetPixel_internal
 #define CloseWindow CloseWindow_internal
