@@ -17,15 +17,19 @@
 #define NETWORK_STOP_BY_PLAYER		0
 #define NETWORK_STOP_BY_ERROR		1
 
+char *getNetworkPlayerName(int);
+
 boolean ConnectToServer(char *, int);
 void SendToServer_PlayerName(char *);
 void SendToServer_ProtocolVersion(void);
 void SendToServer_NrWanted(int);
+void SendToServer_LevelFile(void);
 void SendToServer_StartPlaying(void);
 void SendToServer_PausePlaying(void);
 void SendToServer_ContinuePlaying(void);
 void SendToServer_StopPlaying(int);
 void SendToServer_MovePlayer(byte);
 void HandleNetworking(void);
+void DisconnectFromNetworkServer(void);
 
 #endif
