@@ -22,9 +22,10 @@
 void DrawMainMenuExt(int);
 void DrawAndFadeInMainMenu(int);
 void DrawMainMenu(void);
-void DrawHallOfFame(int);
+void DrawHallOfFame(int, int);
 
-void RedrawSetupScreenAfterFullscreenToggle();
+void RedrawSetupScreenAfterFullscreenToggle(void);
+void RedrawSetupScreenAfterScreenRotation(int);
 
 void HandleTitleScreen(int, int, int, int, int);
 void HandleMainMenu(int, int, int, int, int);
@@ -36,7 +37,12 @@ void HandleSetupScreen(int, int, int, int, int);
 void HandleTypeName(int, Key);
 void HandleGameActions(void);
 
-void CreateScreenGadgets();
-void FreeScreenGadgets();
+void CreateScreenGadgets(void);
+void FreeScreenGadgets(void);
+
+void setHideRelatedSetupEntries(void);
+
+void DumpScreenIdentifiers(void);
+boolean DoScreenAction(int);
 
 #endif	/* SCREENS_H */
