@@ -21,6 +21,12 @@
 /* values for tape properties */
 #define MAX_TAPE_LEN		(1000 * FRAMES_PER_SECOND) /* max.time x fps */
 
+/* values for tape mouse actions */
+#define TAPE_ACTION_LX		0
+#define TAPE_ACTION_LY		1
+#define TAPE_ACTION_BUTTON	2
+#define TAPE_ACTION_UNUSED	3
+
 /* some positions in the video tape control window */
 #define VIDEO_DISPLAY1_XPOS	5
 #define VIDEO_DISPLAY1_YPOS	5
@@ -103,6 +109,8 @@ struct TapeButtonInfo
   struct XY pause;
   struct XY record;
   struct XY play;
+  struct XY insert_solution;
+  struct XY play_solution;
 };
 
 struct TapeSymbolInfo
@@ -182,6 +190,8 @@ struct TapeInfo
   int centered_player_nr_next;
   boolean set_centered_player;
 
+  boolean use_mouse;
+
   struct
   {
     byte action[MAX_PLAYERS];
@@ -202,11 +212,11 @@ struct TapeInfo
 void DrawVideoDisplay(unsigned int, unsigned int);
 void DrawCompleteVideoDisplay(void);
 
-void TapeDeactivateDisplayOn();
+void TapeDeactivateDisplayOn(void);
 void TapeDeactivateDisplayOff(boolean);
 
 void TapeSetDateFromEpochSeconds(time_t);
-void TapeSetDateFromNow();
+void TapeSetDateFromNow(void);
 
 void TapeStartRecording(int);
 void TapeHaltRecording(void);
@@ -223,18 +233,21 @@ unsigned int GetTapeLengthFrames(void);
 unsigned int GetTapeLengthSeconds(void);
 void TapeQuickSave(void);
 void TapeQuickLoad(void);
-void InsertSolutionTape(void);
+
+boolean hasSolutionTape(void);
+boolean InsertSolutionTape(void);
+boolean PlaySolutionTape(void);
 
 void AutoPlayTape(void);
 
-void CreateTapeButtons();
-void FreeTapeButtons();
-void MapTapeEjectButton();
-void MapTapeWarpButton();
-void MapTapeButtons();
-void UnmapTapeButtons();
-void RedrawTapeButtons();
-void RedrawOrRemapTapeButtons();
+void CreateTapeButtons(void);
+void FreeTapeButtons(void);
+void MapTapeEjectButton(void);
+void MapTapeWarpButton(void);
+void MapTapeButtons(void);
+void UnmapTapeButtons(void);
+void RedrawTapeButtons(void);
+void RedrawOrRemapTapeButtons(void);
 
 void HandleTapeButtonKeys(Key);
 
