@@ -40,9 +40,10 @@
 #define DOOR_COPY_BACK		(1 << 4)
 #define DOOR_NO_COPY_BACK	(1 << 5)
 #define DOOR_NO_DELAY		(1 << 6)
-#define DOOR_FORCE_REDRAW	(1 << 7)
-#define DOOR_GET_STATE		(1 << 8)
-#define DOOR_SET_STATE		(1 << 9)
+#define DOOR_FORCE_ANIM		(1 << 7)
+#define DOOR_FORCE_REDRAW	(1 << 8)
+#define DOOR_GET_STATE		(1 << 9)
+#define DOOR_SET_STATE		(1 << 10)
 
 #define DOOR_1			(DOOR_ACTION_1)
 #define DOOR_2			(DOOR_ACTION_2)
@@ -74,53 +75,54 @@ int getLevelFromScreenY(int);
 void DumpTile(int, int);
 void DumpTileFromScreen(int, int);
 
-void DrawMaskedBorder_FIELD();
-void DrawMaskedBorder_DOOR_1();
-void DrawMaskedBorder_DOOR_2();
-void DrawMaskedBorder_DOOR_3();
-void DrawMaskedBorder_ALL();
+void DrawMaskedBorder_FIELD(void);
+void DrawMaskedBorder_DOOR_1(void);
+void DrawMaskedBorder_DOOR_2(void);
+void DrawMaskedBorder_DOOR_3(void);
+void DrawMaskedBorder_ALL(void);
 void DrawMaskedBorder(int);
 void DrawMaskedBorderToTarget(int);
+void DrawTileCursor(int);
 
 void SetDrawtoField(int);
-void RedrawPlayfield();
+void RedrawPlayfield(void);
 void BlitScreenToBitmap_RND(Bitmap *);
 void BlitScreenToBitmap(Bitmap *);
-void BackToFront();
+void BackToFront(void);
 void BackToFront_WithFrameDelay(unsigned int);
 
 void FadeIn(int);
 void FadeOut(int);
-void FadeSetEnterMenu();
-void FadeSetLeaveMenu();
-void FadeSetEnterScreen();
-void FadeSetNextScreen();
-void FadeSetLeaveScreen();
+void FadeSetEnterMenu(void);
+void FadeSetLeaveMenu(void);
+void FadeSetEnterScreen(void);
+void FadeSetNextScreen(void);
+void FadeSetLeaveScreen(void);
 void FadeSetFromType(int);
-void FadeSetDisabled();
-void FadeSkipNextFadeIn();
-void FadeSkipNextFadeOut();
+void FadeSetDisabled(void);
+void FadeSkipNextFadeIn(void);
+void FadeSkipNextFadeOut(void);
 
 Bitmap *getGlobalBorderBitmapFromStatus(int);
 
-void ClearField();
+void ClearField(void);
 void SetWindowBackgroundImageIfDefined(int);
 void SetMainBackgroundImageIfDefined(int);
 void SetDoorBackgroundImageIfDefined(int);
 void SetWindowBackgroundImage(int);
 void SetMainBackgroundImage(int);
 void SetDoorBackgroundImage(int);
-void SetPanelBackground();
+void SetPanelBackground(void);
 void DrawBackground(int, int, int, int);
 void DrawBackgroundForFont(int, int, int, int, int);
 void DrawBackgroundForGraphic(int, int, int, int, int);
-boolean CheckIfGlobalBorderHasChanged();
-void RedrawGlobalBorder();
+boolean CheckIfGlobalBorderHasChanged(void);
+void RedrawGlobalBorder(void);
 
 void MarkTileDirty(int, int);
-void SetBorderElement();
-void FloodFillLevel(int, int, int, short[MAX_LEV_FIELDX][MAX_LEV_FIELDY],
-		    int, int);
+void SetBorderElement(void);
+void FloodFillLevel(int, int, int, short[][MAX_LEV_FIELDY], int, int);
+void FloodFillLevelExt(int, int, int, int, int y, short field[][y], int, int);
 
 void SetRandomAnimationValue(int, int);
 int getGraphicAnimationFrame(int, int);
@@ -157,6 +159,8 @@ void DrawFixedGraphicThruMaskExt(DrawBuffer *, int, int, int, int);
 
 void DrawSizedGraphic(int, int, int, int, int);
 void DrawSizedGraphicExt(DrawBuffer *, int, int, int, int, int);
+void DrawSizedGraphicThruMask(int, int, int, int, int);
+void DrawSizedGraphicThruMaskExt(DrawBuffer *, int, int, int, int, int);
 
 void DrawMiniGraphic(int, int, int);
 void DrawMiniGraphicExt(DrawBuffer *, int, int, int);
@@ -176,7 +180,9 @@ void DrawLevelElement(int, int, int);
 void DrawScreenField(int, int);
 void DrawLevelField(int, int);
 
+void DrawSizedWallParts_MM(int, int, int, int, boolean, int);
 void DrawSizedElement(int, int, int, int);
+void DrawSizedElementThruMask(int, int, int, int);
 void DrawSizedElementOrWall(int, int, int, int, int);
 
 void DrawMiniElement(int, int, int);
@@ -191,6 +197,9 @@ void DrawMiniLevel(int, int, int, int);
 void DrawPreviewLevelInitial(void);
 void DrawPreviewLevelAnimation(void);
 
+void DrawNetworkPlayers(void);
+void ClearNetworkPlayers(void);
+
 void WaitForEventToContinue(void);
 boolean Request(char *, unsigned int);
 void InitGraphicCompatibilityInfo_Doors(void);
@@ -201,11 +210,11 @@ unsigned int GetDoorState(void);
 unsigned int SetDoorState(unsigned int);
 unsigned int MoveDoor(unsigned int);
 
-void DrawSpecialEditorDoor();
-void UndrawSpecialEditorDoor();
+void DrawSpecialEditorDoor(void);
+void UndrawSpecialEditorDoor(void);
 
-void CreateToolButtons();
-void FreeToolButtons();
+void CreateToolButtons(void);
+void FreeToolButtons(void);
 
 int map_element_RND_to_EM(int);
 int map_element_EM_to_RND(int);
@@ -219,6 +228,13 @@ int map_element_RND_to_SP(int);
 int map_element_SP_to_RND(int);
 int map_action_SP_to_RND(int);
 
+int map_element_RND_to_MM(int);
+int map_element_MM_to_RND(int);
+int map_action_MM_to_RND(int);
+int map_sound_MM_to_RND(int);
+int map_mm_wall_element(int);
+int map_mm_wall_element_editor(int);
+
 int get_next_element(int);
 int el_act_dir2img(int, int, int);
 int el_act2img(int, int);
@@ -246,23 +262,28 @@ unsigned int InitRND(int);
 void InitGraphicInfo_EM(void);
 
 void PlayMenuSoundExt(int);
-void PlayMenuSound();
+void PlayMenuSound(void);
 void PlayMenuSoundStereo(int, int);
 void PlayMenuSoundIfLoopExt(int);
-void PlayMenuSoundIfLoop();
+void PlayMenuSoundIfLoop(void);
 void PlayMenuMusicExt(int);
-void PlayMenuMusic();
-void PlayMenuSoundsAndMusic();
-void FadeMenuSoundsAndMusic();
-void PlaySoundActivating();
-void PlaySoundSelecting();
+void PlayMenuMusic(void);
+void PlayMenuSoundsAndMusic(void);
+void FadeMenuSoundsAndMusic(void);
+void PlaySoundActivating(void);
+void PlaySoundSelecting(void);
 
 void SetAnimStatus(int);
 void SetGameStatus(int);
 void SetFontStatus(int);
-void ResetFontStatus();
+void ResetFontStatus(void);
 
-void ToggleFullscreenOrChangeWindowScalingIfNeeded();
-void ChangeViewportPropertiesIfNeeded();
+void SetLevelSetInfo(char *, int);
+
+void ToggleFullscreenOrChangeWindowScalingIfNeeded(void);
+void ChangeViewportPropertiesIfNeeded(void);
+
+boolean CheckIfPlayfieldViewportHasChanged(void);
+boolean CheckIfGlobalBorderOrPlayfieldViewportHasChanged(void);
 
 #endif	/* TOOLS_H */
